@@ -3,16 +3,17 @@ from os import popen
 import socket
 from shutil import copytree, rmtree, ignore_patterns
 import threading
-import requests
 from numpy import random
 from datetime import datetime
 import time
 import subprocess
+from dateutil.parser import parse
+
 
 try:
-    from .utils import is_port_in_use, read_yaml, write_yaml, show_chart, get_results, find_web_port
+    from .utils import is_port_in_use, read_yaml, write_yaml, show_chart, get_results, find_web_port, request_url
 except:
-    from utils import is_port_in_use, read_yaml, write_yaml, show_chart, get_results, find_web_port
+    from utils import is_port_in_use, read_yaml, write_yaml, show_chart, get_results, find_web_port, request_url
 try:
     from .configs import init_directory, folder_name, web_port_default
 except:
@@ -26,7 +27,7 @@ class CreateDirectory:
 
     1. First, it checks if there is existed folder.
     2. Then, imports files by deleting if it is already existed.
-    3. You may reset folcder to default
+    3. You may reset folder to default
 
     ** Notes **
         - Pls. make sure before stop and rerun the process copy the models, data in a safe folder.
@@ -79,22 +80,33 @@ class CreateDirectory:
             print("Files are existed :", self.folder)
 
 
-
-
     def reset_update_config_directory(self, env, reset=False):
         """
             - It allows us to run platform on given folder, otherwise directory will be the local path.
-            - It updates 'directory' variable at "configs.yaml".
-            - When it checks for the 'directory', it won`t be 'None' after all.
+            - It updates 'instace.yaml' at package where it is installed.
+            - When you run package first it checks the instance.yaml
+              where it is installed in the folder you have installed the package.
+
+        ** instance.yaml **
+        Keeps all running and has been run models in same folder. It allows us to restart each platform
+        from where it is ended.
+            -   directory : where user have assigned on AnomalyDetection(path=....)
+            -   absolute_path : It is the path that you have running the platform
+            -   web : This is the port that available for web insterface
+            -   id : random generated id
+            -   active : is the platform active? are services callable?
+            -   start_date : started date of instance
 
         ** Additional Information **
-        When platform is closed or shutdown,
-        directory variable at 'configs.yaml' is updated back to 'None' in order to get default 'configs.yaml' file.
-        If directory variable == None directory will be 'init_directory'.
+            -   When platform is closed or shutdown;
+                instance.yaml is updating. active is being changed to False.
+            -   If there has not been generated instance at absolute path before;
+                It generates new instance with its parameters.
+                Once the instance created, it can be used later when you have start it in same absolute folder.
 
         :param env: docker, local. if docker == True, no need to to update directory be
                     cause whole .py file are also copied to new folder
-        :param reset: reset updates back to normal 'None' value to derectory variable at configs.yaml
+        :param reset: reset updates back to normal 'False' value to directory variable at instance.yaml
         """
         if env != 'docker':
             instances = read_yaml(init_directory, "instance.yaml")
@@ -114,18 +126,13 @@ class CreateDirectory:
                 instances['instances'] += [{'directory': self.folder,
                                             'absolute_path': abspath(""),
                                             'web': find_web_port(),
+                                            'web_host': '127.0.0.1' if env == 'local' else env,
                                             'id': str(random.random()).replace(".", ""),
                                             'active': True,
                                             'start_date': datetime.now()}]
             write_yaml(init_directory, "instance.yaml", instances)
 
 
-            #folder = self.folder
-            #self.configs['directory'] = self.folder if not reset else None
-            #for f in [self.folder, init_directory]:
-            #    write_yaml(join(f, "docs"), "configs.yaml", self.configs)
-
-
 class Configurations:
     """
     This handles the configurations of the Machine Learning Platform. Where to run?
@@ -134,10 +141,10 @@ class Configurations:
     You may also run each platform on nodes or workers.
 
     ****
-    Unless you are runnning on all services on same machine,
+    Unless you are running on all services on same machine,
     you have no specific parameters to assign servers to nodes or worker,
     model services; model_iso_f, model_lstm, model_prophet
-    Machine Learning Service; ml_exeture
+    Machine Learning Service; ml_execute
     ***
 
     Best Features
@@ -157,11 +164,10 @@ class Configurations:
     """
     def __init__(self, path, host='local', remove_existed=False, master_node=True):
         """
-
-        :param path:
-        :param host:
-        :param remove_existed:
-        :param master_node:
+        :param path: folder that user aim to start folder.
+        :param host: local or docker
+        :param remove_existed: if there is a folder which is generated before it removes
+        :param master_node: If it the master node web interface will be initialized.
         """
         self.path = path
         self.host = host
@@ -188,17 +194,19 @@ class Configurations:
         :param service_count: number of service which cpecifically assigned for this configuration.
                               By defaults finds available port for each services.
         """
-        from .configs import conf
+        try:
+            from configs import conf
+        except Exception as e:
+            from .configs import conf
         if self.cd.check_for_directory():
             count = 0
             available_ports = conf('available_ports')
             while len(self.ports) != service_count:
                 if not is_port_in_use(available_ports[count]):
-                    self.ports.append(available_ports[count])
+                    self.ports.append(int(available_ports[count]))
                 count += 1
 
     def check_for_host(self, api_name, count):
-
         if self.cd.check_for_directory():
             if type(self.host) == 'list':
                 return self.host[count]
@@ -207,6 +215,14 @@ class Configurations:
             elif type(self.host) != 'list' and type(self.host) != 'set':
                 return self.host
 
+    def check_for_api(self, apis, api_name, var):
+        acception = True
+        if apis is not None:
+            if api_name in list(apis.keys()):
+                if var in list(apis[api_name].keys()):
+                    acception = False
+        return acception
+
     def create_docker_compose_file(self):
         if self.cd.check_for_directory():
             self.compose_file = read_yaml(self.folder, "docker-compose.yml")
@@ -225,7 +241,7 @@ class Configurations:
             write_yaml(self.folder, "docker-compose.yml", self.compose_file)
 
     def check_for_api_and_host(self, apis=None):
-        if apis is None and self.host not in ['docker', 'local']:
+        if apis is not None and self.host not in ['docker', 'local']:
             if len(apis) == len(self.host):
                 return True
             else:
@@ -234,12 +250,15 @@ class Configurations:
             return  True
 
     def update_api_file(self, apis=None):
-        from .configs import conf
+        try:
+            from configs import conf
+        except Exception as e:
+            from .configs import conf
         self.api_file = read_yaml(conf('docs_main_path'), "apis.yaml")
         if apis is not None:
             if not self.master_node:
                 self.api_file = {a: self.api_file[a] for a in apis}
-            if type(apis) == 'dict':
+            if type(apis) == dict:
                 for a in apis:
                     for p in apis[a]:
                         self.api_file[a][p] = apis[a][p]
@@ -251,18 +270,25 @@ class Configurations:
             self.api_file = {a: self.api_file[a] for a in list(self.api_file.keys()) if a != 'ml_execute'}
         write_yaml(join(self.folder, "docs"), "apis.yaml", self.api_file)
 
-    def create_api_file(self, environment):
-        print(self.host)
+    def create_api_file(self, environment, apis=None):
         if self.cd.check_for_directory():
             self.check_for_ports(service_count=len(self.api_file))
             count = 0
             for s in self.api_file:
-                print("available port for service :", s, " - ", str(self.ports[count]))
-                self.api_file[s]['port'] = self.ports[count]
-                self.api_file[s]['host_create'] = socket.gethostname() if self.host != 'docker' else '0.0.0.0'
-                self.api_file[s]['host'] = socket.gethostname() if self.host != 'docker' else '0.0.0.0'
-                if self.host != 'local' and environment != 'docker':
-                    self.api_file[s]['host'] = self.check_for_host(s, count)
+                if self.check_for_api(apis, s, 'port'):
+                    self.api_file[s]['port'] = self.ports[count]
+                    print("available port for service :", s, " - ", str(self.ports[count]))
+                else:
+                    print("available port for service :", s, " - ", str(self.api_file[s]['port']))
+
+                if self.check_for_api(apis, s, 'host'):
+                    self.api_file[s]['host_create'] = '127.0.0.1' if self.host != 'docker' else '0.0.0.0'
+                    self.api_file[s]['host'] = '127.0.0.1' if self.host != 'docker' else '0.0.0.0'
+
+                if self.check_for_api(apis, s, 'port'):
+                    if self.host != 'local' and environment != 'docker':
+                        self.api_file[s]['host'] = self.check_for_host(s, count)
+
                 count += 1
             self.update_api_yaml_file()
 
@@ -279,13 +305,12 @@ class BuildPlatform:
         from .web.main import web_service_run
         if self.master_node:
             thr = threading.Thread(target=web_service_run)
+            thr.daemon = True
             thr.start()
 
     def docker_env(self):
-        print(join(self.conf.folder, "docker-compose.yml") + " up")
         # stream = popen((join(self.conf.folder, "docker-compose.yml") + " up"))
         result = subprocess.run(["docker-compose", "-f", join(self.conf.folder, "docker-compose.yml"), 'up'], stdout=subprocess.PIPE)
-        print(result.stdout)
         # create web
         self.create_web()
 
@@ -306,8 +331,12 @@ class BuildPlatform:
             self.docker_env()
 
     def down_web(self):
-        from configs import conf
-        requests.post(url='http://0.0.0.0:' + str(conf('web_port')) + '/shutdown')
+        try:
+            from configs import conf
+        except Exception as e:
+            from .configs import conf
+        request_url(url='http://' + conf('web_host') + ':' + str(conf('web_port')) + '/shutdown')
+
 
     def down(self):
         self.down_web()
@@ -316,7 +345,7 @@ class BuildPlatform:
         else:
             for s in self.conf.api_file:
                 api = self.conf.api_file[s]
-                requests.post(url='http://' + api['host'] + ':' + str(api['port']) + '/shutdown')
+                request_url(url='http://' + api['host'] + ':' + str(api['port']) + '/shutdown')
                 time.sleep(2)
 
     def create_data_source(self, **args):
@@ -367,14 +396,16 @@ class AnomalyDetection:
         self.web_port = None
         self.info_dict = {}
         self.jobs = None
+        self.apis = None
 
     def init(self, apis=None):
+        self.apis = None
         if self.conf.check_for_api_and_host(apis=apis):
             self.conf.create_directory()
             if self.env == 'docker':
                 self.conf.create_docker_compose_file()
             self.conf.update_api_file(apis=apis)
-            self.conf.create_api_file(environment=self.env)
+            self.conf.create_api_file(environment=self.env, apis=apis)
             print("Configuration process is completed!!!")
             print("platform is initialized!!!")
         else:
@@ -384,11 +415,15 @@ class AnomalyDetection:
         self.platform = BuildPlatform(conf=self.conf, environment=self.env, master_node=self.conf.master_node)
         self.platform.initialize()
         if self.conf.master_node:
-            from configs import conf
+            try:
+                from configs import conf
+            except Exception as e:
+                from .configs import conf
             self.web_port = conf('web_port')
+            self.web_host = conf('web_host')
             print("platform is up!!!")
             print("*"*5, " WEB APPLICATION ", "*"*5)
-            print('Running on ', 'http://0.0.0.0:' + str(self.web_port) + '/')
+            print('Running on ', 'http://' + self.web_host + ':' + str(self.web_port) + '/')
         else:
             print("platform is up!!!")
             print("Running Services:")
@@ -428,7 +463,6 @@ class AnomalyDetection:
             print("This is not the master node which ml_execute service and web application are running.")
 
     def check_available_apis(self):
-        print(self.conf.api_file)
         return self.conf.api_file
 
     def create_data_source(self,
@@ -466,7 +500,6 @@ class AnomalyDetection:
                     print("please makse sure file format is .", data_source_type, " but ", data_query_path, " is not.")
             elif data_source_type == 'googlebigquery':
                 if db.split(".")[-1] == 'json':
-                    print("yess")
                     connection = self.platform.create_data_source(**self.info_dict)
                     print("Data source is created!!!") if connection else print("Connection is refused!!!")
                 else:
@@ -475,20 +508,17 @@ class AnomalyDetection:
             print("Here are the available data connections: ", ", ".join(sources))
 
     def get_ml_dictionary(self, job={}):
-        print(job)
         asd =  {'description': job.get('description', None),
                 "dates": job.get('dates', None),
                 'groups': job.get('groups', None),
                 'time_indicator': job.get('time_indicator', None),
                 'feature': job.get('feature', None),
                 'days': job.get('days', None)}
-        print(asd)
         return asd
 
     def create_jobs(self, jobs):
         if len(jobs.keys()) > 1:
             for job in jobs:
-                print(jobs[job])
                 if None not in [self.get_ml_dictionary(job=jobs[job])[p] for p in ['dates', 'time_indicator', 'feature', 'days']]:
                     self.platform.create_job(job=job, **self.get_ml_dictionary(job=jobs[job]))
                 else:
@@ -496,19 +526,19 @@ class AnomalyDetection:
                     print({'train': self.get_ml_dictionary({}), 'prediction': self.get_ml_dictionary({})})
                     print("*** "*5)
                     print("Rules of Creating Job / Jobs :")
-                    print("**1. 'time_indicator', 'feature', 'days' are monditory fields.")
-                    print("**2. Jobs are 'train', 'prediction', 'paramter_tuning'.")
-                    print("**3. If you don`t assign any date to train it will directly start assignin as 2 min after the current time.")
-                    print("**4. Prediction 'date' is monditory.")
-                    print("**5. If you don`t assign any date to paramter_tuning it will directly start assignin as 1 day after the  train job date.")
+                    print("**1. 'time_indicator', 'feature', 'days' are manditory fields.")
+                    print("**2. Jobs are 'train', 'prediction', 'parameter_tuning'.")
+                    print("**3. If you don`t assign any date to train \n"
+                          "it will directly start assigning as 2 min after the current time.")
+                    print("**4. Prediction 'date' is manditory.")
+                    print("**5. If you don`t assign any date to paramter_tuning \n"
+                          "it will directly start assigning as 1 day after the  train job date.")
         else:
             self.platform.create_job(job=list(jobs.keys())[0], **self.get_ml_dictionary(job=jobs[list(jobs.keys())[0]]))
-
         print("job is created")
 
     def manage_train(self, stop=False):
         self.platform.run_stop_jobs(job='train', stop=stop)
-
         print("job is created")
 
     def manage_prediction(self, stop=False):
@@ -530,7 +560,10 @@ class AnomalyDetection:
         :return:
         """
         if self.jobs is None:
-            from configs import conf
+            try:
+                from configs import conf
+            except Exception as e:
+                from .configs import conf
             self.jobs = read_yaml(conf('docs_main_path'), "ml_execute.yaml")
         time_indicator = self.jobs['prediction']['execute'][0]['params']['time_indicator']
         feature = self.jobs['prediction']['execute'][0]['params']['feature']
@@ -538,7 +571,19 @@ class AnomalyDetection:
         results = results if filter is None else results.query(filter)
         show_chart(results, time_indicator, 'predict', feature, is_bar_chart=False)
 
-    def show_anomaly_detection(self):
+    def show_anomaly_detection(self, dates=None):
+        if self.jobs is None:
+            try:
+                from configs import conf
+            except Exception as e:
+                from .configs import conf
+            self.jobs = read_yaml(conf('docs_main_path'), "ml_execute.yaml")
+        time_indicator = self.jobs['prediction']['execute'][0]['params']['time_indicator']
+        feature = self.jobs['prediction']['execute'][0]['params']['feature']
+        results = get_results(time_indicator)
+        if dates is not None:
+            results = results[results[time_indicator] >= parse(dates[0]) & results[time_indicator] <= parse(dates[1])]
+        show_chart(results, time_indicator, 'predict', feature, is_bar_chart=False)
         print("")
 
 
